from __future__ import absolute_import, division, print_function, unicode_literals

PARTY_STATUSES = {'Active', 'Inactive', 'Superseded'}
PARTY_TYPES = {'AssetManager', 'Broker', 'Company', 'Exchange', 'Fund', 'GovernmentAgency',
               'Individual', 'Organisation', 'Party'}

